import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

Page {
    title: i18n.tr("Addressbook")
    visible: false

    Component.onCompleted: loadAddresses()
    Connections {
        target: addressbook
        onAddressbookChanged: loadAddresses()
    }

    function loadAddresses() {
        addressbookModel.clear()
        var addressArray = addressbook.getAddressbook()
        for (var i=0; i < addressArray.length; i++)
            addressbookModel.append({"address": addressArray[i]})
    }

    ListModel {
        id: addressbookModel
        objectName: "addressbookModel"
    }

    Column {
        anchors.fill: parent

        Repeater {
            model: addressbookModel

            delegate: ListItem.Standard {
                removable: true
                confirmRemoval: true
                text: addressbook.getAddressLabel(address)
                onItemRemoved: addressbook.removeFromAddressbook(address)
            }
        }

        ListItem.Standard {
            text: i18n.tr("No addresses saved")
            visible: addressbookModel.count < 1
            control: Button {
                text: i18n.tr("Add")
                width: units.gu(10)
                onClicked: PopupUtils.open(addAddressDialog)
            }
        }
    }

    tools: ToolbarItems {
        id: toolbarAddressbook

        ToolbarButton {
            action: Action {
                text: i18n.tr("Add address")
                iconSource: Qt.resolvedUrl('image://theme/add')
                onTriggered: {
                    toolbarAddressbook.opened = false
                    PopupUtils.open(addAddressDialog)
                }
            }
        }
    }

    Component {
        id: addAddressDialog
        Dialog {
            id: dialog
            title: i18n.tr("Add new address")
            text: i18n.tr("Enter an address with associated label.")

            TextField {
                id: addressField
                placeholderText: i18n.tr("Enter address to add")
                onTextChanged: color = "black"
            }
            TextField {
                id: labelField
                color: addressbook.canAddToAddressbook(text) ? "black" : "red"
                placeholderText: i18n.tr("Enter a label for this address")
            }
            Button {
                text: i18n.tr("Add")
                enabled: { addressbook.canAddToAddressbook(labelField.text) &&
                           labelField.text && addressField.text }
                onClicked: {
                    if (!wallets.addressIsValid(addressField.text))
                        addressField.color = "red"
                    else {
                        for (var i=0; i < addressbookModel.count; i++) {
                            if (addressbookModel.get(i).address == addressField.text) {
                                PopupUtils.open(addressExistsErrorDialog)
                                return
                            }
                        }

                        addressbook.setAddressLabel(addressField.text,
                                               labelField.text)
                        addressbook.addToAddressbook(addressField.text)
                        PopupUtils.close(dialog)
                    }
                }
            }
            Button {
                text: i18n.tr("Cancel")
                gradient: UbuntuColors.greyGradient
                onClicked: PopupUtils.close(dialog)
            }
        }
    }

    Component {
        id: addressExistsErrorDialog
        Dialog {
            id: dialog
            title: i18n.tr("Already in addressbook")
            text: i18n.tr("This address is already saved in the addressbook.")

            Button {
                text: i18n.tr("Close")
                onClicked: PopupUtils.close(dialog)
            }
        }
    }
}
