import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

ListItem.Empty {
    anchors { left: parent.left; right: parent.right }
    height: child.height
    showDivider: false
    removable: payments.count > 1
    onItemRemoved: payments.remove(index)

    // >0 when user clicks an autocomplete suggestion.
    property int selectionMade: 0

    UbuntuShape {
        id: child
        anchors { left: parent.left; right: parent.right }

        color: "lightblue"
        radius: "medium"
        state: (addressField.activeFocus && !selectionMade ?
                "expanded" : "collapsed")

        ListModel {
            id: autocompleteModel
            objectName: "autocompleteModel"
        }

        Column {
            anchors {
                top: parent.top
                left: parent.left
                right: parent.right
                topMargin: units.gu(1)
                leftMargin: units.gu(1)
                rightMargin: units.gu(1)
            }
            spacing: units.gu(1)

            Label {
                id: label
                text: i18n.tr("Pay")
            }

            Row {
                spacing: units.gu(1)

                TextField {
                    id: amountField
                    width: units.gu(16)
                    hasClearButton: false
                    inputMethodHints: Qt.ImhFormattedNumbersOnly
                    validator: DoubleValidator {bottom: 0}
                    onTextChanged: payments.setProperty(index, "amount", text)
                }

                Button {
                    id: popoverButton
                    width: units.gu(7)
                    height: amountField.height
                    text: denomination
                    onClicked: PopupUtils.open(popoverComponent, popoverButton)
                }
            }

            Label {
                text: i18n.tr("To")
            }

            TextField {
                id: addressField
                anchors { left: parent.left; right: parent.right }
                color: !addressIsValid ? "red" : "black"
                onTextChanged: {
                    // Will be 1 when selection is made.
                    // Should reset to 0 on next change.
                    if (selectionMade > 1)
                        selectionMade = 0
                    else if (selectionMade > 0)
                        selectionMade += 1

                    autocompleteModel.clear()
                    var label;
                    var addressArray = addressbook.getAddressbook()
                    for (var i=0; i < addressArray.length &&
                                  autocompleteModel.count < 3; i++) {
                        label = addressbook.getAddressLabel(addressArray[i])
                        if (text && label.toLowerCase().indexOf(text.toLowerCase()) !== -1)
                            autocompleteModel.append({"label": label})
                    }

                    payments.setProperty(index, "addressIsValid", true)
                    payments.setProperty(index, "address", text)
                }
            }

            Column {
                id: autocompleteCol
                anchors { left: parent.left; right: parent.right;
                          margins: units.gu(1) }
                height: units.gu(8)
                spacing: units.gu(1)

                Repeater {
                    model: autocompleteModel

                    Label {
                        anchors { left: parent.left; right: parent.right }
                        height: units.gu(2)
                        text: label
                        color: "black"
                        MouseArea {
                            anchors.fill: parent
                            onClicked: {
                                selectionMade = 1
                                addressField.text = text
                            }
                        }
                    }
                }
            }
        }

        Component {
            id: popoverComponent
            ActionSelectionPopover {
                id: popover

                delegate: ListItem.Standard {
                    text: action.text
                }

                actions: ActionList {
                    Action {
                        text: "BTC"
                        onTriggered: {
                            payments.setProperty(index, "denomination", text);
                            PopupUtils.close(popover)
                        }
                    }
                    Action {
                        text: "mBTC"
                        onTriggered: {
                            payments.setProperty(index, "denomination", text);
                            PopupUtils.close(popover)
                        }
                    }
                    Action {
                        text: "μBTC"
                        onTriggered: {
                            payments.setProperty(index, "denomination", text);
                            PopupUtils.close(popover)
                        }
                    }
                }
            }
        }

        states: [
            State {
                name: "collapsed"
                PropertyChanges {
                    target: child
                    height: units.gu(18)
                }
                PropertyChanges {
                    target: autocompleteCol
                    opacity: 0
                }
            },
            State {
                name: "expanded"
                PropertyChanges {
                    target: child
                    height: units.gu(18) + autocompleteCol.height
                }
                PropertyChanges {
                    target: autocompleteCol
                    opacity: 1
                }
            }
        ]

        transitions: [
            Transition {
                from: "collapsed"
                PropertyAnimation {
                    target: child
                    property: "height"
                    duration: UbuntuAnimation.BriskDuration
                    easing: UbuntuAnimation.StandardEasing
                }
                PropertyAnimation {
                    target: autocompleteCol
                    property: "opacity"
                    duration: UbuntuAnimation.BriskDuration
                    easing: UbuntuAnimation.StandardEasing
                }
            },
            Transition {
                from: "expanded"
                PropertyAnimation {
                    target: child
                    property: "height"
                    duration: UbuntuAnimation.BriskDuration
                    easing: UbuntuAnimation.StandardEasing
                }
                PropertyAnimation {
                    target: autocompleteCol
                    property: "opacity"
                    duration: UbuntuAnimation.BriskDuration
                    easing: UbuntuAnimation.StandardEasing
                }
            }
        ]
    }
}
