import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

import "btcSupport.js" as Script

Item{
    function openSheet(parent) {
        parentSheet = parent

        var amount = new Array()
        var address = new Array()
        for (var i=0; i < transactions.count; i++) {
            amount[i] = transactions.get(i).amount
            address[i] = transactions.get(i).address
        }

        if (wallets.wallet(0).createRequest(amount, address) >= 0) {
            PopupUtils.open(sheet)
            return true
        }
        else
            return false
    }

    property DefaultSheet parentSheet

    Component {
        id: sheet

        DefaultSheet {
            id: innerSheet
            contentsHeight: root.height
            contentsWidth: root.width
            title: i18n.tr("Confirm Transaction")

            Column {
                anchors { left: parent.left; right: parent.right }
                spacing: units.gu(3)

                Row {
                    spacing: units.gu(1)

                    Label {
                        text: i18n.tr("Total")
                        fontSize: "x-large"
                    }
                    Label {
                        text: Script.displayFullAmount(wallets.wallet(0).getTotal())
                        color: "#707070"
                        font.bold: true
                        fontSize: "x-large"
                    }
                }

                Row {
                    spacing: units.gu(1)

                    Label {
                        text: i18n.tr("Transaction fee:")
                    }
                    Label {
                        text: Script.displayFullAmount(wallets.wallet(0).getFee())
                    }
                }

                Column {
                    Label {
                        text: i18n.tr("Amount remaining")
                        fontSize: "large"
                    }

                    ListItem.Standard {
                        text: Script.displayRoundedAmount(
                            wallets.wallet(0).balance - wallets.wallet(0).getTotal())
                        iconSource: Qt.resolvedUrl("../images/wallet.png")
                        iconFrame: false
                        showDivider: false
                    }
                }

                Button {
                    anchors { left: parent.left; right: parent.right }
                    text: i18n.tr("Send")
                    onClicked: {
                        wallets.wallet(0).sendTransaction()

                        // Save any new labels
                        var elem
                        for (var i=0; i < transactions.count; i++) {
                            elem = transactions.get(i)
                            if (elem.label)
                                addressbook.setAddressLabel(elem.address, elem.label)
                            if (elem.addToAddressbook)
                                addressbook.addToAddressbook(elem.address)
                        }

                        payments.clear()
                        payments.append(
                            {"amount": "", "denomination": "mBTC", "address": ""})
                        PopupUtils.close(parentSheet)
                        PopupUtils.close(innerSheet)
                        tabs.selectedTabIndex = 3
                    }
                }
            }
        }
    }
}
