import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

import "btcSupport.js" as Script

Item{
    ListModel {
        id: transactions
        objectName: "transactions"
    }

    // Use string to avoid limitations of integers going out of range.
    property string totalSatoshies

    /** Elements should contain 'amount' in satoshies, and 'address'. */
    function createTransaction(transactionsArray) {
        transactions.clear()
        var amounts = new Array()
        for (var i=0; i < transactionsArray.length; i++) {
            transactions.append(transactionsArray[i])
            transactions.setProperty(i, "label", "")
            transactions.setProperty(i, "addToAddressbook", false)
            amounts[i] = transactionsArray[i].amount
        }
        totalSatoshies = addressbook.displaySum(amounts)
    }

    function openSheet() { PopupUtils.open(sheet) }

    Component {
        id: sheet

        DefaultSheet {
            id: confirmSheet
            contentsHeight: root.height
            contentsWidth: root.width
            title: i18n.tr("Pay for transaction")

            Column {
                anchors { left: parent.left; right: parent.right }
                spacing: units.gu(3)

                Column {
                    id: totalCol
                    anchors { left: parent.left; right: parent.right }

                    Row {
                        spacing: units.gu(1)
                        anchors.horizontalCenter: parent.horizontalCenter
                        Label {
                            id: totalLabel
                            text: i18n.tr("Pay")
                            fontSize: "x-large"
                        }
                        Label {
                            text: Script.displayFullAmount(totalSatoshies)
                            color: "#707070"
                            font.bold: true
                            fontSize: "x-large"
                        }
                    }
                    Label {
                        visible: totalSatoshies < onebtc
                        text: {
                            var amount = Script.displayFullBitcoin(
                                totalSatoshies)

                            return ("(" + amount + ")")
                        }
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.leftMargin: totalLabel.width
                        fontSize: "medium"
                    }
                }

                Repeater {
                    model: transactions
                    delegate: Column {
                        anchors { left: parent.left; right: parent.right }
                        spacing: units.gu(0.5)

                        Column {
                            Row {
                                spacing: units.gu(1)
                                Label {
                                    id: payLabel
                                    text: i18n.tr("Send")
                                    fontSize: "large"
                                }
                                Label {
                                    text: { Script.displayFullAmount(amount) }
                                    color: "#707070"
                                    font.bold: true
                                    fontSize: "large"
                                }
                            }
                            Label {
                                visible: amount < onebtc
                                text: {
                                    var amount_str = Script.displayFullBitcoin(
                                        amount.toString())

                                    return ("(" + amount_str + ")")
                                }
                                anchors.left: parent.left
                                anchors.leftMargin: payLabel.width
                                fontSize: "medium"
                            }
                        }
                        Row {
                            spacing: units.gu(1)
                            Label {
                                id: toLabel
                                text: i18n.tr("To")
                                fontSize: "large"
                            }
                            Label {
                                text: addressbook.getAddressLabel(address)
                                color: "#707070"
                                font.bold: true
                                fontSize: "large"
                            }
                        }
                        TextField {
                            visible: !addressbook.hasAddressLabel(address)
                            anchors { left: parent.left; right: parent.right;
                                      leftMargin: toLabel.width }
                            color: { (addToAddressbook &&
                                      !addressbook.canAddToAddressbook(text)) ?
                                      "red" : "black" }
                            placeholderText: i18n.tr(
                                "Enter a label for this address")
                            onTextChanged: transactions.setProperty(
                                index, "label", text)
                        }
                        Row {
                            visible: !addressbook.hasAddressLabel(address)
                            anchors { left: parent.left; right: parent.right;
                                      leftMargin: toLabel.width + units.gu(2) }
                            spacing: units.gu(1)

                            CheckBox {
                                onCheckedChanged: transactions.setProperty(
                                    index, "addToAddressbook", checked)
                            }
                            Label {
                                anchors.verticalCenter: parent.verticalCenter
                                text: i18n.tr("Add to addressbook")
                            }
                        }
                    }
                }

                Column {
                    anchors { left: parent.left; right: parent.right }

                    ListItem.Empty {
                        height: label.height + units.gu(1)
                        Label {
                            id: label
                            text: i18n.tr("Pay with")
                            fontSize: "large"
                        }
                    }

                    ListItem.SingleValue {
                        text: i18n.tr("Wallet")
                        value: Script.displayRoundedAmount(wallets.wallet(0).balance)
                        iconSource: Qt.resolvedUrl("../images/wallet.png")
                        iconFrame: false
                        enabled: parseInt(wallets.wallet(0).balance) >= totalSatoshies
                        onClicked: {
                            var element
                            for (var i=0; i < transactions.count; i++) {
                                element = transactions.get(i)
                                if (element.addToAddressbook) {
                                    if (!element.label) {
                                        PopupUtils.open(emptyAddressErrorDialog)
                                        return
                                    }
                                    if (!addressbook.canAddToAddressbook(
                                            element.label)) {
                                        PopupUtils.open(addressbookErrorDialog)
                                        return
                                    }
                                }
                            }

                            if (!finalSheet.openSheet(confirmSheet))
                                PopupUtils.open(feeErrorDialog)
                        }
                    }
                }
            }
        }
    }

    Component {
        id: feeErrorDialog
        Dialog {
            id: dialog
            title: i18n.tr("Transaction error")
            text: i18n.tr("An error occurred creating this transaction. " + 
                          "This might be caused by not having enough money " +
                          "to cover the transaction fee.")

            Button {
                text: i18n.tr("Close")
                onClicked: PopupUtils.close(dialog)
            }
        }
    }

    Component {
        id: addressbookErrorDialog
        Dialog {
            id: dialog
            title: i18n.tr("Cannot add to addressbook")
            text: i18n.tr("Labels in the addressbook must be unique.")

            Button {
                text: i18n.tr("Close")
                onClicked: PopupUtils.close(dialog)
            }
        }
    }

    Component {
        id: emptyAddressErrorDialog
        Dialog {
            id: dialog
            title: i18n.tr("Cannot add to addressbook")
            text: i18n.tr("Must give a label to save in addressbook.")

            Button {
                text: i18n.tr("Close")
                onClicked: PopupUtils.close(dialog)
            }
        }
    }

    SendConfirmFinalSheet { id: finalSheet }
}
