import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1

Page {
    ListModel {
        id: payments
        objectName: "payments"

        ListElement {
            amount: ""
            denomination: "mBTC"
            address: ""
            addressIsValid: true
        }
    }

    Flickable {
        anchors {
            fill: parent
            topMargin: units.gu(1)
            leftMargin: units.gu(1)
            rightMargin: units.gu(1)
        }
        contentHeight: col.height

        Column {
            id: col
            anchors { left: parent.left; right: parent.right }
            spacing: units.gu(1)

            Repeater {
                model: payments
                delegate: SendComponent {}
            }

            Button {
                anchors { left: parent.left; right: parent.right }
                text: i18n.tr("Confirm...")
                onClicked: {
                    // Check if addresses are valid.
                    var valid = true;
                    for (var i=0; i < payments.count; i++) {
                        valid &= wallets.addressIsValid(payments.get(i).address)
                        payments.get(i).addressIsValid = wallets.addressIsValid(
                            payments.get(i).address)
                    }

                    if (valid) {
                        var transactions = new Array()
                        var amount;
                        var count;
                        for (var i=0; i < payments.count; i++) {
                            // Number of decimal places needed
                            if (payments.get(i).denomination == "BTC")
                                count = 8
                            else if (payments.get(i).denomination == "mBTC")
                                count = 5
                            else
                                count = 2

                            amount = payments.get(i).amount.split(".")
                            // Catch case of no decimal point
                            if (amount.length < 2)
                                amount[1] = "00"

                            while (amount[1].length < count)
                                amount[1] += "0"

                            // Drop any digits smaller than a satoshie with slice
                            amount = amount[0] + amount[1].slice(0, count)

                            // Lose leading 0's, parseInt doesn't make sense
                            while (amount[0] == "0")
                                amount = amount.slice(1)

                            transactions[i] = {
                                "amount": amount,
                                "address": addressbook.getAddress(
                                    payments.get(i).address)}
                        }

                        confirmSheet.createTransaction(transactions)
                        confirmSheet.openSheet()
                    }
                }
            }
        }
    }

    SendConfirmSheet { id: confirmSheet }

    tools: ToolbarItems {
        id: toolbarSend

        ToolbarButton {
            action: Action {
                text: i18n.tr("Clear")
                iconSource: Qt.resolvedUrl('image://theme/edit-clear')
                onTriggered: {
                    toolbarSend.opened = false
                    payments.clear()
                    payments.append(
                        {"amount": "", "denomination": "BTC", "address": ""})
                }
            }
        }
        ToolbarButton {
            action: Action {
                text: i18n.tr("Add Recipient")
                iconSource: Qt.resolvedUrl('image://theme/add')
                onTriggered: {
                    toolbarSend.opened = false
                    payments.append(
                        {"amount": "", "denomination": "mBTC", "address": ""})
                }
            }
        }
    }
}
