import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem

import "btcSupport.js" as Script

Page {
    Component.onCompleted: loadTransactions()
    Connections {
         target: wallets.wallet(0)
         onBalanceChanged: loadTransactions()
    }

    function loadTransactions() {
        if (wallets.wallet(0).getTransactionCount() == transactions.count)
            return

        transactions.clear()
        for (var i=0; i < wallets.wallet(0).getTransactionCount(); i++)
            transactions.append(
                {"amount": wallets.wallet(0).getTransactionValue(i),
                 "myAddress": wallets.wallet(0).getTransactionMyAddress(i),
                 "otherAddress": wallets.wallet(0).getTransactionTheirAddress(i),
                 "time": new Date(wallets.wallet(0).getTransactionTimestamp(i)),
                 "isSent": wallets.wallet(0).getTransactionStatus(i) == 0})
    }

    ListModel {
        id: transactions
        objectName: "transactions"
    }

    ListView {
        anchors.fill: parent
        model: transactions

        delegate: ListItem.Empty {
            enabled: false
            Component.onCompleted: if (!isSent) sendingUpdate.running = true

            property int sentStatus: 0

            function setStatus() {
                sentStatus = wallets.wallet(0).getTransactionStatus(index)
                if (sentStatus == 0) {
                    sendingUpdate.running = false
                    sentDelay.running = true
                }
            }

            resources: [
                Timer {
                    id: sendingUpdate
                    interval: 250
                    running: false
                    repeat: true
                    onTriggered: setStatus()
                },

                Timer {
                    id: sentDelay
                    interval: 3000
                    running: false
                    repeat: false
                    onTriggered: transactions.setProperty(index, "isSent", true)
                }
            ]

            Row {
                anchors { verticalCenter: parent.verticalCenter;
                          left: parent.left; right: amountLabel.left;
                          leftMargin: units.gu(1) }
                spacing: units.gu(1)

                Row {
                    Image {
                        width: units.gu(4)
                        height: units.gu(4)
                        source: Qt.resolvedUrl("../images/wallet.png")
                    }

                    Icon {
                        anchors.verticalCenter: parent.verticalCenter
                        width: units.gu(3)
                        height: units.gu(3)
                        name: (amount > 0 ? "go-previous" : "go-next")
                    }
                }

                Label {
                    anchors.verticalCenter: parent.verticalCenter
                    text: Script.showTime(time)
                    width: units.gu(7)
                }

                Icon {
                    anchors.verticalCenter: parent.verticalCenter
                    width: activity.width
                    height: activity.width
                    name: "network-offline"
                    visible: !isSent && sentStatus == 2
                }

                ActivityIndicator {
                    id: activity
                    anchors.verticalCenter: parent.verticalCenter
                    running: true
                    visible: !isSent && sentStatus == 1
                }

                Image {
                    anchors.verticalCenter: parent.verticalCenter
                    sourceSize.width: activity.width
                    source: Qt.resolvedUrl("../images/transactionsent.svg")
                    visible: !isSent && sentStatus == 0
                }

                Label {
                    anchors.verticalCenter: parent.verticalCenter
                    text: (amount > 0 ? addressbook.getAddressLabel(myAddress) :
                           addressbook.getAddressLabel(otherAddress))
                    elide: Text.ElideRight
                    width: parent.width - units.gu(16) - (isSent ?
                        0 : activity.width + units.gu(1))
                }
            }

            Label {
                id: amountLabel
                anchors { verticalCenter: parent.verticalCenter;
                          right: parent.right; rightMargin: units.gu(1) }
                text: Script.displayRoundedAmount(Math.abs(amount))
            }
        }
    }
}
