#include "addressbook.h"

QAddressbook::QAddressbook(char* addressfile){
    this->addressfile = addressfile;

    addressMap = new std::unordered_map<std::string,std::string>;

    std::ofstream addressof(addressfile, std::ios_base::out | std::ios_base::app);
    addressof.close();

    std::string line;
    std::string line2;
    std::ifstream address(addressfile, std::ios_base::in | std::ios_base::app);
    if (address.is_open())
    {
        while (std::getline(address,line))
        {
            if (!line.compare("true")){
                std::getline(address,line);
                addressbook.append(QString(line.c_str()));
            }
            getline(address,line2);
            std::pair<std::string, std::string> pair(line,line2);
            addressMap->insert(pair);
        }
        address.close();
    }
}

Q_INVOKABLE QString QAddressbook::getAddressLabel(QString address){
    std::string ret;
    try
    {
        ret = (*addressMap).at(address.toStdString());
    }
    catch (const std::out_of_range& oor)
    {
        return address;
    }
    return QString(ret.c_str());
}

Q_INVOKABLE QString QAddressbook::getAddress(QString label){
    for(int i = 0; i < addressbook.size(); i++)
    {
        if (!QString((*addressMap)[addressbook[i].toStdString()].c_str()).compare(label,Qt::CaseSensitive))
        {
            return addressbook[i];
        }
    }
    for(auto kv : (*addressMap)) {
        if (!QString(kv.second.c_str()).compare(label, Qt::CaseSensitive))
        {
            return QString(kv.first.c_str());
        }
    }
    return label;
}

Q_INVOKABLE void QAddressbook::setAddressLabel(QString address, QString label){
    if (addressbook.contains(address))
    {
        if (!canAddToAddressbook(label))
            return;
    }
    (*addressMap)[address.toStdString()] = label.toStdString();
    updateAddressFile();
}

Q_INVOKABLE bool QAddressbook::hasAddressLabel(QString address){
    try
    {
        (*addressMap).at(address.toStdString());
    }
    catch (const std::out_of_range& oor)
    {
        return false;
    }
    return true;
}

Q_INVOKABLE QList<QString> QAddressbook::getAddressbook(){
    return addressbook;
}

Q_INVOKABLE bool QAddressbook::canAddToAddressbook(QString label){
    for(int i = 0; i < addressbook.size(); i++)
    {
        if (!QString((*addressMap)[addressbook[i].toStdString()].c_str()).compare(label,Qt::CaseSensitive))
        {
            return false;
        }
    }
    return true;
}

Q_INVOKABLE void QAddressbook::addToAddressbook(QString address){
    if (canAddToAddressbook(QString((*addressMap)[address.toStdString()].c_str()))){
        addressbook.append(address);
        updateAddressFile();
    }
}

Q_INVOKABLE void QAddressbook::removeFromAddressbook(QString address){
    addressbook.removeOne(address);
    updateAddressFile();
}

void QAddressbook::updateAddressFile(){
    std::string line;
    std::string line2;

    std::ofstream addressf(addressfile, std::ios_base::out);
    if (addressf.is_open())
    {
        for(auto it = addressMap->begin(); it!= addressMap->end(); ++it) {
            std::string key = it->first;
            std::string value = it->second;
            
            if (addressbook.contains(QString(key.c_str()))){
                 addressf << "true" << "\n";
            }
            addressf << key << "\n";
            addressf << value << "\n";
        }
        addressf.close();
    }
    emit addressbookChanged();
}


Q_INVOKABLE QString QAddressbook::displaySum(QList<int> values){
    int sum = 0;
    for(int i = 0; i < values.count(); i++){
        sum += values[i];
    }
    return QString(std::to_string(sum).c_str());
}
