#include "localwallet.h"

QLocalWallet::QLocalWallet(char* blockfile, char* walletfile, std::string location){
    options.optionString = "-Djava.class.path=java-7-openjdk-amd64:./:wallet/libs/bitcoinj-0.10.3.jar:wallet/libs/slf4j-nop-1.7.5.jar:wallet/libs/slf4j-api-1.7.5.jar:wallet/libs/protobuf-java-2.4.1.jar:wallet/libs/netty-3.7.0.Final.jar:wallet/libs/sc-light-jdk15on-1.47.0.3.jar:wallet/libs/guava-15.0.jar"; 

    vm_args.version = JNI_VERSION_1_6; //JDK version. This indicates version 1.6
    vm_args.nOptions = 1;
    vm_args.options = &options;
    vm_args.ignoreUnrecognized = 0;
    
    JNI_CreateJavaVM(&jvm, (void**)&env, &vm_args);
    jclass cls = env->FindClass("wallet/LocalWallet");
    if(cls == 0)
        std::cerr << "Failed to load class." << std::endl;
    obj = env->AllocObject(cls);

    ubcInit = env->GetMethodID(cls, "<init>", "(Ljava/lang/String;Ljava/lang/String;)V");
    if(ubcInit == 0)
        std::cerr << "Failed to load function: Constructor" << std::endl;

    ubcGetAddress =                 env->GetMethodID(cls, "getAddress",                 "(I)Ljava/lang/String;");
    ubcAddAddress =                 env->GetMethodID(cls, "addAddress",                 "()Ljava/lang/String;");
    ubcGetAddressCount =            env->GetMethodID(cls, "getAddressCount",            "()I");
    
    ubcGetTransactionCount =        env->GetMethodID(cls, "getTransactionCount",        "()I");
    ubcGetTransactionStatus =        env->GetMethodID(cls, "getTransactionStatus",        "(I)I");
    ubcGetTransactionValue =        env->GetMethodID(cls, "getTransactionValue",        "(I)I");
    ubcGetTransactionMyAddress =    env->GetMethodID(cls, "getTransactionMyAddress",    "(I)Ljava/lang/String;");
    ubcGetTransactionTheirAddress = env->GetMethodID(cls, "getTransactionTheirAddress", "(I)Ljava/lang/String;");
    ubcGetTransactionTimestamp =    env->GetMethodID(cls, "getTransactionTimestamp",    "(I)Ljava/lang/String;");
    ubcGetTransactionConfidence =   env->GetMethodID(cls, "getTransactionConfidence",   "(I)I");
    ubcGetTransactionHash =         env->GetMethodID(cls, "getTransactionHash",         "(I)Ljava/lang/String;");
    ubcGetValue =                   env->GetMethodID(cls, "getValue",                   "()I");
    
    ubcSendCoins =                  env->GetMethodID(cls, "sendCoins",                  "(ILjava/lang/String;)V");
    ubcCreateRequest =              env->GetMethodID(cls, "createRequest",              "([Ljava/lang/String;[Ljava/lang/String;)I");
    ubcGetFee =                     env->GetMethodID(cls, "getFee",                     "()I");
    ubcGetTotal =                   env->GetMethodID(cls, "getTotal",                   "()I");
    ubcSendTransaction =            env->GetMethodID(cls, "sendTransaction",            "()V");
    
    ubcAddressIsValid =             env->GetMethodID(cls, "addressIsValid",             "(Ljava/lang/String;)Z");
    
	//Start bitcoin client
    jstring jblockfile = env->NewStringUTF(blockfile);
    jstring jwalletFile = env->NewStringUTF(walletfile);
    env->CallVoidMethod(obj, ubcInit, jblockfile, jwalletFile);
}

long QLocalWallet::balance(){
    return (int)env->CallObjectMethod(obj, ubcGetValue);
}


Q_INVOKABLE int QLocalWallet::getAddressCount(){
    return (int)env->CallObjectMethod(obj, ubcGetAddressCount);
}

Q_INVOKABLE QString QLocalWallet::getAddress(int num){
    jint result = env->CallObjectMethod(obj, ubcGetAddress, num);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE QString QLocalWallet::addAddress(){
    jint result = (int)env->CallObjectMethod(obj, ubcAddAddress);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE int QLocalWallet::getTransactionCount(){
    return (int)env->CallObjectMethod(obj, ubcGetTransactionCount);
}

Q_INVOKABLE int QLocalWallet::getTransactionStatus(int tx){
    return (int)env->CallObjectMethod(obj, ubcGetTransactionStatus, tx);
}

Q_INVOKABLE int QLocalWallet::getTransactionValue(int tx){
    return (int)env->CallObjectMethod(obj, ubcGetTransactionValue, tx);
}

Q_INVOKABLE QString QLocalWallet::getTransactionMyAddress(int tx){
    jint result = env->CallObjectMethod(obj, ubcGetTransactionMyAddress, tx);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE QString QLocalWallet::getTransactionTheirAddress(int tx){
    jint result = env->CallObjectMethod(obj, ubcGetTransactionTheirAddress, tx);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE QString QLocalWallet::getTransactionTimestamp(int tx){
    jint result = env->CallObjectMethod(obj, ubcGetTransactionTimestamp, tx);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE int QLocalWallet::getTransactionConfidence(int tx){
    return (int)env->CallObjectMethod(obj, ubcGetTransactionConfidence, tx);
}

Q_INVOKABLE QString QLocalWallet::getTransactionHash(int tx){
    jint result = env->CallObjectMethod(obj, ubcGetTransactionHash, tx);
    if (result)
    {
        const char *resultcstring = env->GetStringUTFChars(result, 0);
        char c[100];
        sprintf(c, "%s",resultcstring);
        env->ReleaseStringUTFChars(result, resultcstring);
        env->DeleteLocalRef(result);
        return c;
    }
    return QString("No address found.");
}

Q_INVOKABLE QList<QString> QLocalWallet::getAddresses(){
    QList<QString> list;
    for(int i = 0;i < getAddressCount(); i++)
        list.append(getAddress(i));
    return list;
}

Q_INVOKABLE QList<int> QLocalWallet::getTransactionValues(){
    QList<int> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionValue(i));
    return list;
}

Q_INVOKABLE QList<QString> QLocalWallet::getTransactionTheirAddresses(){
    QList<QString> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionTheirAddress(i));
    return list;
}

Q_INVOKABLE QList<QString> QLocalWallet::getTransactionTimestamps(){
    QList<QString> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionTimestamp(i));
    return list;
}

Q_INVOKABLE QList<QString> QLocalWallet::getTransactionMyAddresses(){
    QList<QString> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionMyAddress(i));
    return list;
}

Q_INVOKABLE QList<int> QLocalWallet::getTransactionConfidences(){
    QList<int> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionConfidence(i));
    return list;
}

Q_INVOKABLE QList<QString> QLocalWallet::getTransactionHashes(){
    QList<QString> list;
    for(int i = 0;i < getTransactionCount(); i++)
        list.append(getTransactionHash(i));
    return list;
}


Q_INVOKABLE void QLocalWallet::sendCoins(int amount, QString address){
    jstring jstr = env->NewStringUTF(address.toStdString().c_str());
    env->CallObjectMethod(obj, ubcSendCoins, amount, jstr);
}

Q_INVOKABLE QString QLocalWallet::createRequest(QList<QString> amount, QList<QString> address){
    jobjectArray addrarr = env->NewObjectArray(amount.size(), env->FindClass("java/lang/String"), env->NewStringUTF(""));
    for(int i = 0; i < amount.size(); i++){
        env->SetObjectArrayElement(addrarr, i, env->NewStringUTF(address[i].toStdString().c_str()));
    }
    jobjectArray amountrarr = env->NewObjectArray(amount.size(), env->FindClass("java/lang/String"), env->NewStringUTF(""));
    for(int i = 0; i < amount.size(); i++){
        env->SetObjectArrayElement(amountrarr, i, env->NewStringUTF(amount[i].toStdString().c_str()));
    }
    int ret = (int)env->CallObjectMethod(obj, ubcCreateRequest, amountrarr, addrarr);
    return QString(std::to_string(ret).c_str());
}

Q_INVOKABLE int QLocalWallet::getFee(){
    return (int)env->CallObjectMethod(obj, ubcGetFee);
}

Q_INVOKABLE int QLocalWallet::getTotal(){
    return (int)env->CallObjectMethod(obj, ubcGetTotal);
}

Q_INVOKABLE void QLocalWallet::sendTransaction(){
    env->CallObjectMethod(obj, ubcSendTransaction);
    emit balanceChanged();
}

Q_INVOKABLE bool QLocalWallet::addressIsValid(QString address){
    jstring jstr = env->NewStringUTF(address.toStdString().c_str());
    jboolean b = env->CallObjectMethod(obj, ubcAddressIsValid, jstr);
    return b;
}
