package wallet; /**
 * Auther: Michael
 * Date: 9/11/13
 * Time: 10:58 AM
 */

import com.google.bitcoin.core.*;
import com.google.bitcoin.discovery.DnsDiscovery;
import com.google.bitcoin.params.MainNetParams;
import com.google.bitcoin.store.BlockStoreException;
import com.google.bitcoin.store.SPVBlockStore;
import com.google.bitcoin.store.UnreadableWalletException;

import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

import static com.google.common.base.Preconditions.checkNotNull;

public class LocalWallet{
    static private SPVBlockStore blockStore;
    static private BlockChain blockChain;
    static private Wallet wallet;
    static private PeerGroup peerGroup;
    private MainNetParams params;
    private File walletFile;
    private File blockfile;
    private String blockfilename;
    private String walletFilename;
    private Wallet.SendRequest request = null;

    public String getAddress(int addressnum){
        return wallet.getKeys().get(addressnum).toAddress(params).toString();
    }

    public int getAddressCount(){
        return wallet.getKeys().size();
    }

    public String addAddress(){
        wallet.addKey(new ECKey());
        return wallet.getKeys().get(wallet.getKeys().size()-1).toAddress(params).toString();
    }

    public int getValue(){
        return wallet.getBalance(Wallet.BalanceType.ESTIMATED).intValue();
    }

    public int getTransactionCount() {
        return wallet.getTransactions(true).size();
    }

    private Transaction[] sortedTransactions()
    {
        Transaction[] transactions = Arrays.copyOf(wallet.getTransactions(true).toArray(),
                wallet.getTransactions(true).toArray().length,
                Transaction[].class);
        for (int i = 0; i < transactions.length; i++)
        {
            for (int j = i; j < transactions.length; j++)
            {
                if (transactions[i].getUpdateTime().compareTo(transactions[j].getUpdateTime()) < 0)
                {
                    Transaction temp = transactions[i];
                    transactions[i] = transactions[j];
                    transactions[j] = temp;
                }
            }
        }
        return transactions;
    }

    public int getTransactionStatus(int tx) {
        if ((sortedTransactions()[tx]).getConfidence().numBroadcastPeers() > 0 || (sortedTransactions()[tx]).getConfidence().getDepthInBlocks()>0)
            return 0;
        if (peerGroup.numConnectedPeers() > 0)
            return 1;
        else
            return 2;
    }

    public int getTransactionValue(int tx) {
        try {
            return (sortedTransactions()[tx]).getValue(wallet).intValue();
        } catch (ScriptException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return 0;
    }

    public String getTransactionMyAddress(int tx) {
        List<TransactionOutput> transactionOutputs = (sortedTransactions()[tx]).getOutputs();
        TransactionOutput myOutput = null;
        TransactionOutput theirOutput = null;
        if (transactionOutputs != null) {
            for (TransactionOutput transactionOutput : transactionOutputs) {
                if (transactionOutput != null && transactionOutput.isMine(wallet)) {
                    myOutput = transactionOutput;
                }
                if (transactionOutput != null && !transactionOutput.isMine(wallet)) {
                    theirOutput = transactionOutput;
                }
            }
        }
        try {
            return myOutput.getScriptPubKey().getToAddress(params).toString();
        } catch (ScriptException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return "";
    }

    public String getTransactionTheirAddress(int tx) {
        List<TransactionOutput> transactionOutputs = (sortedTransactions()[tx]).getOutputs();
        TransactionOutput myOutput = null;
        TransactionOutput theirOutput = null;
        if (transactionOutputs != null) {
            for (TransactionOutput transactionOutput : transactionOutputs) {
                if (transactionOutput != null && transactionOutput.isMine(wallet)) {
                    myOutput = transactionOutput;
                }
                if (transactionOutput != null && !transactionOutput.isMine(wallet)) {
                    theirOutput = transactionOutput;
                }
            }
        }
        try {
            return theirOutput.getScriptPubKey().getToAddress(params).toString();
        } catch (ScriptException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }
        return "";
    }

    public int getTransactionConfidence(int tx) {
        return (sortedTransactions()[tx]).getConfidence().getDepthInBlocks();
    }

    public String getTransactionHash(int tx){
        return (sortedTransactions()[tx]).getHashAsString();
    }

    public String getTransactionTimestamp(int tx){
        SimpleDateFormat df = new SimpleDateFormat("EEE, d MMM yyyy HH:mm:ss Z");
        return df.format((sortedTransactions()[tx]).getUpdateTime());
    }

    public boolean addressIsValid(String address){
        try {
            new Address(params, address);
        } catch (AddressFormatException e) {
            return false;
        }
        return true;
    }

    public LocalWallet(String blockfilestring, String walletFilestring) throws InterruptedException, BlockStoreException, IOException, AddressFormatException {
        params = MainNetParams.get();
        walletFile = new File(walletFilestring);
        boolean makeWallet = true;
        if (walletFile.exists()){
            try{
                wallet = Wallet.loadFromFile(walletFile);
                makeWallet = false;
            } catch (UnreadableWalletException e ){}
        }
        if (makeWallet){ // Make wallet if load failled wallet.
            wallet = new Wallet(params);
            wallet.addKey(new ECKey());
            wallet.saveToFile(walletFile);
        }
        wallet.autosaveToFile(walletFile, 500, TimeUnit.MILLISECONDS, null);
        wallet.allowSpendingUnconfirmedTransactions();

        blockfile = new File(blockfilestring);
        blockStore = new SPVBlockStore(params, blockfile);
        blockChain = new BlockChain(params, wallet, blockStore);
        peerGroup = new PeerGroup(params,blockChain);
        peerGroup.setUserAgent("ubc.Ubc", "1.0");
        peerGroup.addPeerDiscovery(new DnsDiscovery(params));
        peerGroup.addWallet(wallet);
        peerGroup.startAsync();
        peerGroup.startBlockChainDownload(new DownloadListener());
    }


    private void sendCoins(int amount, String address) throws AddressFormatException {
        final Wallet.SendResult sendResult = wallet.sendCoins(peerGroup, new Address(params,address), new BigInteger(String.valueOf(amount)));
        checkNotNull(sendResult);
    }


    private int createRequest(String[] amount, String[] address) throws AddressFormatException, VerificationException, ExecutionException, InterruptedException {
        Transaction transaction = new Transaction(params);
        for (int i = 0; i < amount.length; i++){
            transaction.addOutput(new BigInteger(amount[i]),  new Address(params,address[i]));
        }
        request = Wallet.SendRequest.forTx(transaction);
        wallet.completeTx(request);
        int fee = 0;
        if (request.fee == null)
            fee = -1;
        else
            fee = request.fee.intValue();
        return fee;
    }

    private int getFee() throws AddressFormatException, VerificationException, ExecutionException, InterruptedException {
        int fee;
        if (request.fee == null)
            fee = -1;
        else
            fee = request.fee.intValue();
        return fee;
    }

    private int getTotal() throws AddressFormatException, VerificationException, ExecutionException, InterruptedException {
        return request.tx.getValueSentFromMe(wallet).intValue()-request.tx.getValueSentToMe(wallet).intValue();
    }

    private void sendTransaction() throws VerificationException, ExecutionException, InterruptedException {
        if (request != null && request.fee != null)
        {
            wallet.commitTx(request.tx);
            peerGroup.broadcastTransaction(request.tx);
            request = null;
        }
    }
}
